import numpy as np
import math
import matplotlib.pyplot as plt
from scipy.integrate import solve_ivp

# Definition of the Frenet-Serret differential equations
def frenet_serret(s, y, kappa_func, tau_func):
    T = y[0:3] # Tangent vector
    N = y[3:6] # Principal normal vector
    B = y[6:9] # Binormal vector
    r = y[9:12] # Position vector of the curve

    kappa = kappa_func(s)
    tau = tau_func(s)
    dT_ds = kappa * N
    dN_ds = -kappa * T + tau * B
    dB_ds = -tau * N
    dr_ds = T # Position of the curve

    return np. concatenate ([dT_ds, dN_ds, dB_ds, dr_ds])

# Reconstruction and plotting
def reconstruct_and_plot(kappa_func, tau_func, s_span=(0, 100),
                         initial_conditions=None, resolution= 10000):
    if initial_conditions is None:
        T0 = [1, 0, 0] # Tangent vector
        N0 = [0, 1, 0] # Principal normal vector
        B0 = [0, 0, 1] # Binormal vector
        r0 = [0, 0, 0] # Initial position of the curve
        initial_conditions = T0 + N0 + B0 + r0 # List instead of array

    s_eval = np.linspace(s_span[0], s_span[1], resolution)

    # Solve using Runge-Kutta method
    solution = solve_ivp(frenet_serret, s_span, initial_conditions,
                         args= (kappa_func, tau_func),
                         t_eval=s_eval, method= 'RK45')

    r= solution.y [9:12] # Position of the curve
    x, y, z =r[0], r[1], r[2]
              
    # Check if the torsion is zero
    is_planar = np.allclose([tau_func(s) for s in s_eval], 0, atol=1e-10)

    # Plot for planar curve
    if is_planar:
        plt.figure(figsize= (10, 10))
        plt.plot(x, y, label='Example of curve', color='blue')
        plt.title('Parametric Plot of $(X(s),Y(s))$')
        plt.xlabel('X(t)')
        plt.ylabel('Y(t)')
        plt.grid(True)
        plt.axis('equal')
        plt.legend()
        plt.show()
    else:
        # 3D Plot
        fig = plt.figure(figsize=(10,10))
        ax=fig.add_subplot(111, projection='3d')
        ax.plot(x, y, z, label='Example of curve', color='blue')
        ax.set_title('Parametric Plot of $(X(s),Y(s),Z(s))$')

        # Set axis limits
        ax.set_xlim(0, 1)
        ax.set_ylim(0, 1)
        ax.set_zlim(0, 1)
        ax.set_xlabel('X(s)')
        ax.set_ylabel('Y(s)')
        ax.set_zlabel('Z(s)')
        ax.legend()
        plt.show()

kappa_func = lambda s: 2*s #Curvature
tau_func = lambda s: 0.5 #Torsion
                 
reconstruct_and_plot(kappa_func, tau_func, s_span=(0, 100))
