import numpy as np # Library for numerical computations
import matplotlib.pyplot as plt # Library for plotting
from scipy.integrate import solve_ivp # Function to solve initial value problems for ODES
import time

# Define the right-hand side of the ODE for c(t)
def dc_dt(t, c):
    return np.cos(t**2)

# Define the right-hand side of the ODE for s(t)
def ds_dt(t, s) :
    return np.sin(t**2)

# Function to solve ODEs and generate the clothoid plot
def solve_and_plot():
    # Initial conditions and time range
    c0 = 0 # Initial condition for c(0)
    s0 = 0 # Initial condition for s(0)
    t_span = (0, 100) # Time range: t∈ [O, 100]
    t_eval = np.linspace(0, 100, 10000) # Time points for evaluation (for plotting)

    # Solve the ODE for c(t) using solve_ivp
    c_solution = solve_ivp(dc_dt, t_span, [c0], method= 'RK45', t_eval=t_eval,
                           rtol=1e-6, atol=1e-8)
    C_values = c_solution.y[0] # Extract the values of c(t)

    # Solve the ODE for s(t) using solve_ivp
    s_solution = solve_ivp(ds_dt, t_span, [s0], method='RK45', t_eval=t_eval,
                           rtol=1e-6, atol=1e-8)
    S_values = s_solution.y[0] # Extract the values of s(t)

# Initialize list to store computation times
computation_times = []

# Run the function 1000 times and record computation times
for _ in range(1000):
    start_time = time.time()
    solve_and_plot()
    end_time = time.time()

    computation_times.append(end_time - start_time)

# Compute the average computation time
average_time = np.mean(computation_times)

# Plot the computation times as points
plt.figure(figsize=(10, 5))
plt.plot(computation_times, 'o' , label="Computation Time per Run", color='blue')
  # Use 'o' for points
plt.axhline(y=average_time, color='red', linestyle='--',
            label=f"Average Time = { average_time:.4f}s")
plt.xlabel('Run Number')
plt.ylabel('Computation Time (seconds)')
plt.title('Computation Time for 1000 Runs')
plt.legend()
plt.grid(True)
plt.savefig('list5.pdf')
plt.show()

# Print the average computation time
print(f"Average computation time: {average_time:.4f} seconds")
