import numpy as np # Library for numerical computations
import matplotlib.pyplot as plt # Library for plotting
from scipy. special import erf # Error function for Fresnel integrals

# Define constants
sqrt_pi = np.sqrt(np.pi)  # Square root of pi
sqrt_i = np.sqrt(1j)      # Square root of imaginary unit i
sqrt_neg_i = np.sqrt(-1j) # Square root of -i
# Compute Fresnel integral C(z)
def fresnel_C (z):
    """
    Compute the Fresnel cosine integral C(z) using complex representations.
    Parameters:
    z: ndarray
        Input values (can be complex or real)
    Returns:
    C(z): ndarray
        Fresnel cosine integral values
    """
    return (sqrt_pi/4)*(sqrt_neg_i*erf(sqrt_i*z) + sqrt_i*erf(sqrt_neg_i*z))

# Compute Fresnel integral S(z)
def fresnel_S (z):
    """
    Compute the Fresnel sine integral S(z) using complex representations.
    Parameters:
    z: ndarray
        Input values (can be complex or real)
    Returns :
    S(z): ndarray
        Fresnel sine integral values
    """
    return (sqrt_pi/4)*(sqrt_i*erf(sqrt_i*z) + sqrt_neg_i*erf(sqrt_neg_i*z))

# Generate the clothoid curve
def generate_clothoid(t_vals):
    """
    Generate x and y values for the clothoid curve.

    Parameters:
    t_vals: ndarray
        Parameter values for the curve
    Returns:
    x_vals, y-vals: tuple of ndarrays
        Real parts of C(t) and S(t), representing the clothoid curve
    """
    C_vals = fresnel_C(t_vals)
    S_vals = fresnel_S(t_vals)
    return C_vals.real, S_vals.real

# Define the range of parameter t
t_vals = np.linspace(0, 100, 10000) # Parameter t (larger values expand the curve

# Generate the clothoid curve
x_vals, y_vals = generate_clothoid(t_vals)
# Plot
plt.figure(figsize=(10, 10))
plt.plot(x_vals, y_vals, label="Clothoid curve by complex number display",
         color="blue")
plt.axhline(0, color="black" , linewidth=1.0, linestyle="--")
plt.axvline(0, color="black" , linewidth=1.0, linestyle="--")
plt.title("Parametric Plot of $(C(t), S(t))$",fontsize=10)
plt.xlabel("$C(t)$", fontsize=10)
plt.ylabel("$S(t)$", fontsize=10)
plt.legend(fontsize=10)
plt.grid(alpha =0.5)
plt.axis("equal")
plt.show()
