import numpy as np # Library for numerical computations
import matplotlib.pyplot as plt # Library for plotting
from scipy.integrate import solve_ivp # Function to solve initial value problems for ODES

# Define the Frenet-Serret equations for planar curves
def frenet_serret(s, y, kappa_func):
    """
    Compute derivatives based on Frenet-Serret equations.
    Parameters:
    s: float
        Curve parameter
    y: ndarray
        State vector [T, N, r], where:
        T: Tangent vector (2D)
        N: Normal vector (2D)
        r: Position vector (2D)
    kappa_func: function
        Curvature as a function of s
    Returns: dyds: ndarray
        Derivative of the state vector
    """
    T = y[0:2] # Tangent vector (2D)
    N = y[2:4] # Normal vector (2D)
    r = y[4:6] # Position vector (2D)

    kappa = kappa_func(s) # Curvature at parameter s

    # Compute derivatives
    dT_ds = kappa * N  # Tangent vector derivative
    dN_ds = -kappa * T # Normal vector derivative
    dr_ds = T # Position vector derivative
    
    return np.concatenate([dT_ds, dN_ds, dr_ds])

# Function to compute and plot the planar curve
def reconstruct_and_plot(kappa_func, s_span=(0, 100), initial_conditions=None,
                         resolution =10000):
    """
    Solve Frenet-Serret equations and plot the planar curve.

    Parameters:
    kappa_func: function
        Curvature as a function of s
    s_span: tuple
        Range of parameter s (start, end)
    initial_conditions: list or None
        Initial values for T (tangent), N (normal), and r (position)
    resolution: int
        Number of points for parameter s
    """
    if initial_conditions is None:
        T0 = [1, 0] # Initial tangent vector
        N0 = [0, 1] # Initial normal vector
        r0 = [0, 0] # Initial position
        initial_conditions = T0 + N0 + r0
        
    s_eval = np.linspace(s_span[0], s_span[1], resolution) # Parameter range

    # Solve Frenet-Serret equations
    solution = solve_ivp(frenet_serret, s_span, initial_conditions,
                         args=(kappa_func,), t_eval=s_eval, method='RK45',
                         rtol=1e-6, atol=1e-8)

    r = solution.y[4:6] # Extract position vectors
    x, y = r[0], r[1] # Separate x and y components

    # Plot
    plt.figure(figsize= (10, 10))
    plt.plot(x, y, label='Clothoid by Frenet Serret formulas', color='blue')
    plt.title('Parametric Plot of $(C(t), S(t))$')
    plt.xlabel('C(t)')
    plt.ylabel('S(t)')
    plt.grid(True)
    plt.axis('equal')
    plt.legend()
    plt.show()

# Define curvature function (linear increase)
kappa_func = lambda s: 2.0 * s

# Generate and plot the curve
reconstruct_and_plot(kappa_func, s_span=(0, 100))
