import numpy as np # Library for numerical computations
import matplotlib.pyplot as plt # Library for plotting
from scipy.integrate import solve_ivp # Function to solve initial value problems for ODES
import time

# Define the Frenet-Serret equations for planar curves
def frenet_serret(s, y, kappa_func):

    T = y[0:2] # Tangent vector (2D)
    N = y[2:4] # Normal vector (2D)
    I = y[4:6] # Position vector (2D)

    kappa = kappa_func(s) # Curvature at parameter s

    # Compute derivatives
    dT_ds = kappa * N # Tangent vector derivative
    dN_ds = -kappa * T # Normal vector derivative
    dr_ds = T # Position vector derivative
    
    return np.concatenate([dT_ds, dN_ds, dr_ds])

# Function to compute and plot the planar curve
def reconstruct_and_plot(kappa_func, s_span=(0, 100), initial_conditions=None,
                         resolution=10000):
    
    if initial_conditions is None:
        T0 = [1, 0] # Initial tangent vector
        N0 = [0, 1] # Initial normal vector
        r0 = [0, 0] # Initial position
        initial_conditions = T0 + N0 + r0
        
    s_eval = np.linspace(s_span[0], s_span[1], resolution) # Parameter range

    # Solve Frenet-Serret equations
    solution = solve_ivp(frenet_serret, s_span, initial_conditions,
                         args =(kappa_func,), t_eval=s_eval,
                         method='RK45', rtol=1e-6, atol=1e-8)
    r = solution.y[4:6] # Extract position vectors
    x, y =r[0], r[1] # Separate x and y components

kappa_func = lambda s: 2.0 * s

# Initialize list to store computation times
computation_times = []

# Run the function 1000 times and record computation times
for _ in range(1000):
    start_time = time.time()
    # Define curvature function (linear increase)
    reconstruct_and_plot(kappa_func, s_span=(0, 100))
    end_time = time.time()
    computation_times.append(end_time - start_time)

# Compute the average computation time
average_time = np.mean(computation_times)

# Plot the computation times as points
plt.figure(figsize = (10, 5))
plt.plot(computation_times, 'o', label="Computation Time per Run", color='blue')
  # Use 'o' for points
plt.axhline(y=average_time, color='red', linestyle='--',
            label=f"Average Time = {average_time:.4f}s")
plt.xlabel('Run Number')
plt.ylabel('Computation Time (seconds)')
plt.title('Computation Time for 1000 Runs')
plt.legend()
plt.grid(True)
plt.savefig('list9.pdf')
plt.show()

# Print the average computation time
print(f"Average computation time: {average_time:.4f} seconds")
